/* network.c
 * 
 *  a collection of general network functions.
 */

#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "extern.h"
#include "defines.h"

/* net_connect(): 
 * accepts							returns
 * hostname (char *)
 * port number (int)
 *											socket file descriptor number
 *
 * this function is just to open a socket
 * to a remote host given the hostname and
 * port number.  It returns the fd number of
 * the open socket.
 */
int net_connect(const char *hostname, const int port) {

	int sockfd, connection, ret;
	int error=0;
	struct hostent *ipaddr;
	struct sockaddr_in servaddr;
	struct timeval tv; 
	fd_set fdw, fdr;

	if (verbose) printf("Connecting to %s:%i\n", hostname, port);
	if (debug) print_debug(4, "net_connect(): hostname=%s, port=%d\n", hostname, port);
	ipaddr=gethostbyname(hostname);
	if (!ipaddr) {
		herror("Fatal");
		exit(1);
	}
	sockfd=socket(PF_INET, SOCK_STREAM, 0);
	fcntl(sockfd, F_SETFL, O_NONBLOCK);

	servaddr.sin_family=ipaddr->h_addrtype;
	servaddr.sin_port=htons(port);
	memcpy((char *) &servaddr.sin_addr.s_addr, ipaddr->h_addr_list[0], ipaddr->h_length);
	connection=connect(sockfd, (struct sockaddr *)&servaddr, sizeof(servaddr));

	tv.tv_sec = 30; 
	tv.tv_usec = 0; 

	if (connection) {
		switch (errno) {
			case EAGAIN: /* we can handle this error */
			case EINPROGRESS: 

				FD_ZERO(&fdw);
				FD_SET(sockfd, &fdw);
				fdr = fdw ;
	
				tv.tv_sec = NET_TIMEOUT; 
				tv.tv_usec = 0; 

				if (select(sockfd + 1, &fdr, &fdw, NULL, &tv ) != 0) {
					/* even if socket is writeable, may be an error, need explicit check */
					if (FD_ISSET(sockfd, &fdr) || FD_ISSET(sockfd, &fdw)) {
						int len = sizeof(error);
						if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (char *)&error, &len) == 0) {
							/* solaris getsockopt() will return -1 if an
 								 * error has occurred  otherwise we must
 								 * explicitly check "error"
							 */
							if (error == 0) { /* no errors on connect */
								if (verbose) printf("Connection Established, testing....\n");

								/* set 30 second tiemouts on read/writes */
								tv.tv_sec = NET_DTIMEOUT; 
								tv.tv_usec = 0; 

								setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)); 
								setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)); 
														
								break; /* let's get out of here ... */
							}
						}
					}
				}
				/* Fall through to default */
			default: 
				if (verbose) fprintf(stderr, "Timed Out connection.\n");
				close(sockfd); /* close socket, and (maybe) try again */
				sockfd=0;
				break;
		}
	}
	return sockfd;
}

int scan_for_dot (char *msg) {

	int t;

	t=strlen(msg);
	msg+=t;
	msg--;

	if (debug) print_debug(5, "scan_for_dot(): %d %d %d %d %d\n", *(msg - 4), *(msg - 3),
									*(msg - 2), *(msg - 1), *msg);
	return ((*(msg - 4) == '\r') && (*(msg - 3) == '\n') && (*(msg - 2) == '.')
									&& (*(msg - 1) == '\r') && (*msg == '\n'));

}
												
	 
char *net_read(int sockfd) {	

	int num, r;
	char *str;

	fd_set rdfs;
	struct timeval tv;
	int ret, timeout, i;
	char *buf, *ptr;
	
	unsigned char t[1];


	/* clear string contents and initialise all to null. Good and safe practise */	
	str=calloc(sizeof(char), NET_BUFFER+1);	

	FD_ZERO(&rdfs);
	FD_SET(sockfd, &rdfs);

	timeout = 45;
	tv.tv_sec = timeout;
	tv.tv_usec = 0;

	ret=select((sockfd+1), &rdfs, NULL, NULL, &tv);
	if (ret) {
		num=recv(sockfd, str, NET_BUFFER-1,0);

		if (!num && !ret) {
			perror("An Error Occurred");
		}
	}

	if (!num)	return str; /* safely returned something worth while. */	


	if(debug==5) {
		printf("Debug[5]: net_read() ");
		for(r=0;r<num;r++) {
			t[0]=*(str+r);
			printf("%hhi ", t[0]);
		}
		printf("\n");
	}

	/* return the damn stuff.  */	
	return str;
}

int net_send(int sockfd, char *str) {
	
	int r;

	if(debug) print_debug(5, "Sending: %s", str);
	if (!send(sockfd, str, strlen(str), 0)) {
		perror("Error sending to network");
		exit(1);
	}

}

int net_nsend(int sockfd, char *str, int len) {
	
	int r;
	unsigned char t[1];

	if(debug==5) {
		printf("Debug[5]: net_nsend() ");
		for(r=0;r<len;r++) {
			t[0]=*(str+r);
			printf("%hhi ", t[0]);
		}
		printf("\n");
	}
	if (!send(sockfd, str, len, 0)) {
		perror("Error sending to network");
		exit(1);
	}

}
char *getip(const char *hostname) {

	int sockfd, connection;
	struct hostent *ipaddr;
	struct sockaddr_in servaddr;

	ipaddr=gethostbyname(hostname);
	if (!ipaddr) {
		herror("Fatal");
		exit(1);
	}
	return ipaddr->h_addr_list[0];
}
